@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, unicode_literals
-
 from . import layout # noqa
 from . import links # noqa
 from . import pane # noqa
@@ -10,10 +8,10 @@ from . import widgets # noqa
 from .config import config, panel_extension as extension, __version__ # noqa
 from .depends import bind, depends # noqa
 from .interact import interact # noqa
-from .io import ipywidget, serve, state # noqa
+from .io import _jupyter_server_extension_paths, ipywidget, serve, state # noqa
 from .layout import ( # noqa
-    Accordion, Card, Row, Column, WidgetBox, Tabs, Spacer, 
-    GridSpec, GridBox
+    Accordion, Card, Column, GridSpec, GridBox, FlexBox, Tabs, Row,
+    Spacer, WidgetBox
 )
 from .pane import panel, Pane # noqa
 from .param import Param # noqa
