@@ -2,11 +2,13 @@
 Defines callbacks to be executed on a thread or by scheduling it
 on a running bokeh server.
 """
-from __future__ import absolute_import, division, unicode_literals
-
+import asyncio
+import inspect
 import time
 import param
 
+from bokeh.io import curdoc as _curdoc
+
 from ..util import edit_readonly
 from .state import state
 
@@ -40,7 +42,7 @@ class PeriodicCallback(param.Parameterized):
         Toggles whether the periodic callback is currently running.""")
 
     def __init__(self, **params):
-        super(PeriodicCallback, self).__init__(**params)
+        super().__init__(**params)
         self._counter = 0
         self._start_time = None
         self._cb = None
@@ -65,11 +67,14 @@ class PeriodicCallback(param.Parameterized):
             self.stop()
             self.start()
 
-    def _periodic_callback(self):
+    async def _periodic_callback(self):
         with edit_readonly(state):
             state.busy = True
         try:
-            self.callback()
+            if inspect.isasyncgenfunction(self.callback) or inspect.iscoroutinefunction(self.callback):
+                await self.callback()
+            else:
+                self.callback()
         finally:
             with edit_readonly(state):
                 state.busy = False
@@ -88,6 +93,9 @@ class PeriodicCallback(param.Parameterized):
         """
         return self._counter
 
+    def _cleanup(self, session_context):
+        self.stop()
+
     def start(self):
         """
         Starts running the periodic callback.
@@ -101,13 +109,17 @@ class PeriodicCallback(param.Parameterized):
             finally:
                 self._updating = False
         self._start_time = time.time()
-        if state.curdoc and state.curdoc.session_context:
+        if state.curdoc:
             self._doc = state.curdoc
             self._cb = self._doc.add_periodic_callback(self._periodic_callback, self.period)
         else:
             from tornado.ioloop import PeriodicCallback
-            self._cb = PeriodicCallback(self._periodic_callback, self.period)
+            self._cb = PeriodicCallback(lambda: asyncio.create_task(self._periodic_callback()), self.period)
             self._cb.start()
+        try:
+            state.on_session_destroyed(self._cleanup)
+        except Exception:
+            pass
 
     def stop(self):
         """
@@ -122,7 +134,17 @@ class PeriodicCallback(param.Parameterized):
         self._counter = 0
         self._timeout = None
         if self._doc:
-            self._doc.remove_periodic_callback(self._cb)
-        else:
+            if self._doc._session_context:
+                self._doc.callbacks.remove_session_callback(self._cb)
+            else:
+                self._doc.callbacks._session_callbacks.remove(self._cb)
+        elif self._cb:
             self._cb.stop()
         self._cb = None
+        doc = self._doc or _curdoc()
+        if doc:
+            doc.callbacks.session_destroyed_callbacks = {
+                cb for cb in doc.callbacks.session_destroyed_callbacks
+                if cb is not self._cleanup
+            }
+            self._doc = None
