@@ -14,19 +14,19 @@ function ID() {
 
 export class AcePlotView extends PanelHTMLBoxView {
   model: AcePlot
-  protected _ace: any
   protected _editor: any
   protected _langTools: any
+  protected _modelist: any
   protected _container: HTMLDivElement
 
   initialize(): void {
     super.initialize()
-    this._ace = (window as any).ace
     this._container = div({
       id: ID(),
       style: {
         width: "100%",
-        height: "100%"
+        height: "100%",
+        zIndex: 0,
       }
     })
   }
@@ -36,6 +36,8 @@ export class AcePlotView extends PanelHTMLBoxView {
     this.connect(this.model.properties.code.change, () => this._update_code_from_model())
     this.connect(this.model.properties.theme.change, () => this._update_theme())
     this.connect(this.model.properties.language.change, () => this._update_language())
+    this.connect(this.model.properties.filename.change, () => this._update_filename())
+    this.connect(this.model.properties.print_margin.change, () => this._update_print_margin())
     this.connect(this.model.properties.annotations.change, () => this._add_annotations())
     this.connect(this.model.properties.readonly.change, () => {
       this._editor.setReadOnly(this.model.readonly)
@@ -47,16 +49,19 @@ export class AcePlotView extends PanelHTMLBoxView {
     if (!(this._container === this.el.childNodes[0]))
       this.el.appendChild(this._container)
       this._container.textContent = this.model.code
-      this._editor = this._ace.edit(this._container.id)
-      this._editor.setTheme("ace/theme/" + `${this.model.theme}`)
-      this._editor.session.setMode("ace/mode/" + `${this.model.language}`)
-      this._editor.setReadOnly(this.model.readonly)
-      this._langTools = this._ace.require('ace/ext/language_tools')
+      this._editor = (window as any).ace.edit(this._container.id)
+      this._langTools = (window as any).ace.require('ace/ext/language_tools')
+      this._modelist = (window as any).ace.require("ace/ext/modelist")
       this._editor.setOptions({
         enableBasicAutocompletion: true,
         enableSnippets: true,
         fontFamily: "monospace", //hack for cursor position
       });
+      this._update_theme()
+      this._update_filename()
+      this._update_language()
+      this._editor.setReadOnly(this.model.readonly)
+      this._editor.setShowPrintMargin(this.model.print_margin);
       this._editor.on('change', () => this._update_code_from_editor())
   }
 
@@ -65,18 +70,31 @@ export class AcePlotView extends PanelHTMLBoxView {
       this._editor.setValue(this.model.code)
   }
 
+  _update_print_margin(): void {
+    this._editor.setShowPrintMargin(this.model.print_margin);
+  }
+
   _update_code_from_editor(): void {
     if(this._editor.getValue() !=  this.model.code){
       this.model.code = this._editor.getValue()
     }
   }
 
-  _update_theme(): void{
+  _update_theme(): void {
     this._editor.setTheme("ace/theme/" + `${this.model.theme}`)
   }
 
+  _update_filename(): void {
+    if (this.model.filename) {
+      const mode = this._modelist.getModeForPath(this.model.filename).mode
+      this.model.language = mode.slice(9)
+    }
+  }
+
   _update_language(): void{
-    this._editor.session.setMode("ace/mode/" + `${this.model.language}`)
+    if (this.model.language != null) {
+      this._editor.session.setMode("ace/mode/" + `${this.model.language}`)
+    }
   }
 
   _add_annotations(): void{
@@ -87,7 +105,6 @@ export class AcePlotView extends PanelHTMLBoxView {
     super.after_layout()
     this._editor.resize()
   }
-
 }
 
 export namespace AcePlot {
@@ -95,8 +112,10 @@ export namespace AcePlot {
   export type Props = HTMLBox.Props & {
     code: p.Property<string>
     language: p.Property<string>
+    filename: p.Property<string>
     theme: p.Property<string>
     annotations: p.Property<any[]>
+    print_margin: p.Property<boolean> 
     readonly: p.Property<boolean>
   }
 }
@@ -115,15 +134,17 @@ export class AcePlot extends HTMLBox {
   static init_AcePlot(): void {
     this.prototype.default_view = AcePlotView
 
-    this.define<AcePlot.Props>({
-      code:        [ p.String            ],
-      language:    [ p.String,  'python' ],
-      theme:       [ p.String,  'chrome' ],
-      annotations: [ p.Array,   []       ],
-      readonly:    [ p.Boolean, false    ]
-    })
+    this.define<AcePlot.Props>(({Any, Array, Boolean, String}) => ({
+      code:         [ String,       '' ],
+      filename:     [ String           ],
+      language:     [ String           ],
+      theme:        [ String, 'chrome' ],
+      annotations:  [ Array(Any),   [] ],
+      readonly:     [ Boolean,   false ],
+      print_margin: [ Boolean,   false ]
+    }))
 
-    this.override({
+    this.override<AcePlot.Props>({
       height: 300,
       width: 300
     })
