@@ -4,7 +4,8 @@ Defines custom ECharts bokeh model to render Vega json plots.
 from bokeh.core.properties import Any, Dict, Enum, String
 from bokeh.models import LayoutDOM
 
-from ..util import classproperty, bundled_files
+from ..io.resources import bundled_files
+from ..util import classproperty
 
 
 class ECharts(LayoutDOM):
@@ -14,8 +15,8 @@ class ECharts(LayoutDOM):
     """
 
     __javascript_raw__ = [
-        "https://cdn.jsdelivr.net/npm/echarts@4.8.0/dist/echarts.min.js",
-        "https://cdn.jsdelivr.net/npm/echarts-gl@1.1.1/dist/echarts-gl.min.js"
+        "https://cdn.jsdelivr.net/npm/echarts@5.0.2/dist/echarts.min.js",
+        "https://cdn.jsdelivr.net/npm/echarts-gl@2.0.2/dist/echarts-gl.min.js"
     ]
 
     @classproperty
@@ -29,10 +30,9 @@ class ECharts(LayoutDOM):
         }
 
     __js_require__ = {
-        'baseUrl': 'https://cdn.jsdelivr.net/npm/',
         'paths': {
-            "echarts":  "echarts@4.8.0/dist/echarts.min",
-            "echarts-gl": "echarts-gl@1.1.1/dist/echarts-gl.min.js"
+            "echarts":  "https://cdn.jsdelivr.net/npm/echarts@5.0.2/dist/echarts.min",
+            "echarts-gl": "https://cdn.jsdelivr.net/npm/echarts-gl@2.0.2/dist/echarts-gl.min.js"
         },
         'exports': {}
     }
