@@ -1,3 +1,4 @@
+import {isArray} from "@bokehjs/core/util/types"
 import {HTMLBox} from "@bokehjs/models/layouts/html_box"
 import {div} from "@bokehjs/core/dom"
 import {Enum} from "@bokehjs/core/kinds"
@@ -5,29 +6,113 @@ import * as p from "@bokehjs/core/properties";
 import {ColumnDataSource} from "@bokehjs/models/sources/column_data_source";
 import {TableColumn} from "@bokehjs/models/widgets/tables"
 
+import {debounce} from  "debounce"
+
 import {transform_cds_to_records} from "./data"
 import {PanelHTMLBoxView, set_size} from "./layout"
 
 
 declare const Tabulator: any;
 
+function find_group(key: any, value: string, records: any[]): any {
+  for (const record of records) {
+    if (record[key] == value)
+      return record
+  }
+  return null
+}
+
+function summarize(grouped: any[], columns: any[], aggregators: string[], depth: number = 0): any {
+  const summary: any = {}
+  if (grouped.length == 0)
+    return summary
+  const agg = aggregators[depth]
+  for (const group of grouped) {
+    const subsummary = summarize(group._children, columns, aggregators, depth+1)
+    for (const col in subsummary) {
+      if (isArray(subsummary[col]))
+        group[col] = subsummary[col].reduce((a: any, b: any) => a + b, 0) / subsummary[col].length
+      else
+        group[col] = subsummary[col]
+    }
+    for (const column of columns.slice(1)) {
+      const val = group[column.field]
+      if (column.field in summary) {
+        const old_val = summary[column.field]
+        if (agg === 'min')
+          summary[column.field] = Math.min(val, old_val)
+        else if (agg === 'max')
+          summary[column.field] = Math.max(val, old_val)
+        else if (agg === 'sum')
+          summary[column.field] = val + old_val
+        else if (agg === 'mean') {
+          if (isArray(summary[column.field]))
+            summary[column.field].push(val)
+          else
+            summary[column.field] = [old_val, val]
+        }
+      } else
+        summary[column.field] = val
+    }
+  }
+  return summary
+}
+
+function group_data(records: any[], columns: any[], indexes: string[], aggregators: any): any[] {
+  const grouped = []
+  const index_field = columns[0].field
+  for (const record of records) {
+    const value = record[indexes[0]]
+    let group = find_group(index_field, value, grouped)
+    if (group == null) {
+      group = {_children: []}
+      group[index_field] = value
+      grouped.push(group)
+    }
+    let subgroup = group
+    const groups: any = {}
+    for (const index of indexes.slice(1)) {
+      subgroup = find_group(index_field, record[index], subgroup._children)
+      if (subgroup == null) {
+        subgroup = {_children: []}
+        subgroup[index_field] = record[index]
+        group._children.push(subgroup)
+      }
+      groups[index] = group
+      for (const column of columns.slice(1))
+        subgroup[column.field] = record[column]
+      group = subgroup
+    }
+    for (const column of columns.slice(1))
+      subgroup[column.field] = record[column.field]
+  }
+  const aggs = []
+  for (const index of indexes)
+    aggs.push((index in aggregators) ? aggregators[index] : 'sum')
+  summarize(grouped, columns, aggs)
+  return grouped
+}
+
+
 // The view of the Bokeh extension/ HTML element
 // Here you can define how to render the model as well as react to model changes or View events.
 export class DataTabulatorView extends PanelHTMLBoxView {
   model: DataTabulator;
   tabulator: any;
-  _tabulator_cell_updating: boolean=false;
-  _selection_updating: boolean=false;
+  _tabulator_cell_updating: boolean=false
+  _data_updating: boolean = true
+  _selection_updating: boolean =false
   _styled_cells: any[] = []
+  _styles: any;
   _initializing: boolean
 
   connect_signals(): void {
     super.connect_signals()
 
-    const {configuration, layout, columns, theme, theme_url, groupby} = this.model.properties;
+    const {configuration, layout, columns, theme, groupby} = this.model.properties;
     this.on_change([configuration, layout, columns, groupby], () => this.render_and_resize())
 
-    this.on_change([theme, theme_url], () => this.setCSS())
+    this.on_change([theme], () => this.setCSS())
 
     this.connect(this.model.properties.download.change, () => {
       const ftype = this.model.filename.endsWith('.json') ? "json" : "csv"
@@ -55,6 +140,7 @@ export class DataTabulatorView extends PanelHTMLBoxView {
     })
 
     this.connect(this.model.properties.styles.change, () => {
+      this._styles = this.model.styles
       this.updateStyles()
     })
 
@@ -81,19 +167,13 @@ export class DataTabulatorView extends PanelHTMLBoxView {
     if (wait)
       return
     this._initializing = true
-    const container = div({class: "pnx-tabulator"});
+    this._styles = this.model.styles
+    const container = div({class: "pnx-tabulator"})
     set_size(container, this.model)
-    let configuration = this.getConfiguration();
+    let configuration = this.getConfiguration()
 
     this.tabulator = new Tabulator(container, configuration)
 
-    // Patch the ajax request method
-    this.tabulator.modules = {...this.tabulator.modules}
-    const ajax = this.tabulator.modules.ajax
-    this.tabulator.modules.ajax.sendRequest = () => {
-      this.requestPage(ajax.params.page, ajax.params.sorters)
-    }
-
     // Swap pagination mode
     if (this.model.pagination === 'remote') {
       this.tabulator.options.pagination = this.model.pagination
@@ -114,12 +194,26 @@ export class DataTabulatorView extends PanelHTMLBoxView {
     this.el.appendChild(container)
   }
 
+  tableInit(view: DataTabulatorView, tabulator: any): void {
+    // Patch the ajax request and page data parsing methods
+    const ajax = tabulator.modules.ajax
+    ajax.sendRequest = () => {
+      return view.requestPage(ajax.params.page, ajax.params.sorters)
+    }
+    tabulator.modules.page._parseRemoteData = () => {}
+  }
+
   requestPage(page: number, sorters: any[]): Promise<void> {
     return new Promise((resolve: any, reject: any) => {
       try {
-        this.model.page = page || 1
-        this.model.sorters = sorters
-        resolve({data: [], last_page: this.model.max_page})
+        if (page != null && sorters != null) {
+          if (this._data_updating)
+            this._data_updating = false
+          else
+            this.model.sorters = sorters
+          this.model.page = page || 1
+        }
+        resolve([])
       } catch(err) {
         reject(err)
       }
@@ -159,27 +253,47 @@ export class DataTabulatorView extends PanelHTMLBoxView {
 
   getConfiguration(): any {
     const pagination = this.model.pagination == 'remote' ? 'local': (this.model.pagination || false)
+    // Only use selectable mode if explicitly requested otherwise manually handle selections
+    let selectable = this.model.select_mode === 'toggle' ? true : NaN
+    const that = this
     let configuration = {
       ...this.model.configuration,
       index: "_index",
+      nestedFieldSeparator: false,
+      selectable: selectable,
+      tableBuilding: function() { that.tableInit(that, this) },
       renderComplete: () => this.renderComplete(),
       rowSelectionChanged: (data: any, rows: any) => this.rowSelectionChanged(data, rows),
+      rowClick: (e: any, row: any) => this.rowClicked(e, row),
       cellEdited: (cell: any) => this.cellEdited(cell),
       columns: this.getColumns(),
       layout: this.getLayout(),
-      ajaxSorting: true,
       pagination: pagination,
       paginationSize: this.model.page_size,
       paginationInitialPage: 1,
+      selectableCheck: (row: any) => {
+        const selectable = this.model.selectable_rows
+        return (selectable == null) || (selectable.indexOf(row._row.data._index) >= 0)
+      },
+      tooltips: (cell: any) => {
+        return  cell.getColumn().getField() + ": " + cell.getValue();
+      },
+      scrollVertical: debounce(() => {
+        this.updateStyles()
+      }, 50, false)
     }
-    if (pagination)
+    if (pagination) {
       configuration['ajaxURL'] = "http://panel.pyviz.org"
+      configuration['ajaxSorting'] = true
+    }
     const cds: any = this.model.source;
     let data: any[]
     if (cds === null || (cds.columns().length === 0))
-      data = transform_cds_to_records(cds, true)
-    else
       data = []
+    else
+      data = transform_cds_to_records(cds, true)
+    if (configuration.dataTree)
+      data = group_data(data, this.model.columns, this.model.indexes, this.model.aggregators)
     return {
       ...configuration,
       "data": data,
@@ -197,8 +311,13 @@ export class DataTabulatorView extends PanelHTMLBoxView {
             group_columns.push({...col})
           columns.push({...column, columns: group_columns})
         } else {
+	  if (column.formatter === "rowSelection") {
+	    column.cellClick = (_: any, cell: any) => {
+	      cell.getRow().toggleSelect();
+	    }
+	  }
           columns.push({...column})
-        }
+	}
     }
     for (const column of this.model.columns) {
       let tab_column: any = null
@@ -232,13 +351,16 @@ export class DataTabulatorView extends PanelHTMLBoxView {
           tab_column.formatter = "tickCross"
         else {
           tab_column.formatter = (cell: any) => {
-            return column.formatter.doFormat(cell.getRow(), cell, cell.getValue(), null, null)
+            const formatted = column.formatter.doFormat(cell.getRow(), cell, cell.getValue(), null, null)
+            if (column.formatter.type === 'HTMLTemplateFormatter')
+              return formatted
+            const node = div()
+            node.innerHTML = formatted
+            return node.children[0].innerHTML
           }
         }
       }
 
-      tab_column.editable = () => this.model.editable
-
       const editor: any = column.editor
       const ctype = editor.type
       if (tab_column.editor != null) {
@@ -258,9 +380,10 @@ export class DataTabulatorView extends PanelHTMLBoxView {
       } else if (ctype === "SelectEditor") {
         tab_column.editor = "select"
         tab_column.editorParams = {values: editor.options}
-      } else {
+      } else if (editor != null && editor.default_view != null) {
         tab_column.editor = (cell: any, onRendered: any, success: any, cancel: any) => this.renderEditor(column, cell, onRendered, success, cancel)
       }
+      tab_column.editable = () => (this.model.editable && (editor.default_view != null))
       if (config_columns == null)
         columns.push(tab_column)
     }
@@ -301,11 +424,16 @@ export class DataTabulatorView extends PanelHTMLBoxView {
   // Update table
 
   setData(): void {
-    const data = transform_cds_to_records(this.model.source, true);
+    let data = transform_cds_to_records(this.model.source, true);
+    if (this.model.configuration.dataTree)
+      data = group_data(data, this.model.columns, this.model.indexes, this.model.aggregators)
+    this._data_updating = true
     if (this.model.pagination != null)
       this.tabulator.rowManager.setData(data, true, false)
-    else
-      this.tabulator.setData(data);
+    else {
+      this.tabulator.setData(data)
+      this._data_updating = false
+    }
     this.freezeRows()
     this.updateSelection()
   }
@@ -337,20 +465,24 @@ export class DataTabulatorView extends PanelHTMLBoxView {
     let old_node: any = null
     const links = document.getElementsByTagName("link")
     const dist_index = this.model.theme_url.indexOf('dist/')
+    const start_url = this.model.theme_url.slice(0, dist_index)
     for (const link of links) {
-      if (link.href.startsWith(this.model.theme_url.slice(0, dist_index))) {
+      if (link.href.indexOf(start_url) >= 0) {
         old_node = link
         break
       }
     }
 
-    let parent_node = document.getElementsByTagName("head")[0]
     if (old_node != null) {
-      if (old_node.href == css)
+      if (old_node.href.endsWith(css))
         return false
-      if (old_node.parentNode != null)
-        parent_node = old_node.parentNode
+      else {
+        old_node.href = css
+        setTimeout(() => this.render_and_resize(), 100)
+        return true
+      }
     }
+    let parent_node = document.getElementsByTagName("head")[0]
 
     const css_node: any = document.createElement('link')
     css_node.type = 'text/css'
@@ -359,10 +491,6 @@ export class DataTabulatorView extends PanelHTMLBoxView {
     css_node.href = css
 
     css_node.onload = () => {
-      if (old_node != null && old_node.parentNode != null) {
-        parent_node = old_node.parentNode
-        parent_node.removeChild(old_node)
-      }
       this.render_and_resize()
     }
     parent_node.appendChild(css_node)
@@ -373,10 +501,10 @@ export class DataTabulatorView extends PanelHTMLBoxView {
     for (const cell_el of this._styled_cells)
       cell_el.cssText = ""
     this._styled_cells = []
-    if (this.model.styles == null || this.tabulator.getDataCount() == 0)
+    if (this._styles == null || this.tabulator == null || this.tabulator.getDataCount() == 0)
       return
-    for (const r in this.model.styles) {
-      const row_style = this.model.styles[r]
+    for (const r in this._styles) {
+      const row_style = this._styles[r]
       const row = this.tabulator.getRow(r)
       if (!row)
         continue
@@ -390,26 +518,29 @@ export class DataTabulatorView extends PanelHTMLBoxView {
         this._styled_cells.push(element)
         element.cssText = ""
         for (const s of style) {
-          if (!s.includes(':'))
+          let prop, value
+          if (isArray(s))
+            [prop, value] = s
+          else if (!s.includes(':'))
             continue
-          const [prop, value] = s.split(':')
+          else
+            [prop, value] = s.split(':')
           element.style.setProperty(prop, value.trimLeft())
         }
       }
     }
+    const styles = this._styles
+    this.model.styles = {}
+    this._styles = styles
   }
 
   addData(): void {
-    const rows = this.tabulator.rowManager.getRows();
+    const rows = this.tabulator.rowManager.getRows()
     const last_row = rows[rows.length-1]
-
-    let data = transform_cds_to_records(this.model.source, true);
-    this.tabulator.setData(data);
-    if (this.model.follow) {
-      this.tabulator.scrollToRow((last_row.data._index || 0), "top", false);
-    }
-    this.freezeRows()
-    this.updateSelection()
+    const start = ((last_row?.data._index) || 0)
+    this.setData()
+    if (this.model.follow && last_row)
+      this.tabulator.scrollToRow(start, "top", false)
   }
 
   updateOrAddData(): void {
@@ -433,12 +564,13 @@ export class DataTabulatorView extends PanelHTMLBoxView {
   }
 
   setMaxPage(): void {
-    this.tabulator.setMaxPage(Math.max(this.model.page, this.model.max_page))
-    this.tabulator.modules.page._setPageButtons()
+    this.tabulator.setMaxPage(this.model.max_page)
+    if (this.tabulator.modules.page.pagesElement)
+      this.tabulator.modules.page._setPageButtons()
   }
 
   setPage(): void {
-    this.tabulator.setPage(this.model.page)
+    this.tabulator.setPage(Math.min(this.model.max_page, this.model.page))
   }
 
   setPageSize(): void {
@@ -450,20 +582,72 @@ export class DataTabulatorView extends PanelHTMLBoxView {
       return
 
     const indices = this.model.source.selected.indices;
+    const current_indices: any = this.tabulator.getSelectedData().map((row: any) => row._index)
+    if (JSON.stringify(indices) == JSON.stringify(current_indices))
+      return
     this._selection_updating = true
     this.tabulator.deselectRow()
     this.tabulator.selectRow(indices)
+    // This actually places the selected row at the top of the table
+    this.tabulator.scrollToRow(indices[0], "bottom", false)
     this._selection_updating = false
   }
 
   // Update model
 
-  rowSelectionChanged(data: any, _: any): void {
-    if (this._selection_updating || this._initializing)
+  rowClicked(e: any, row: any) {
+    if (this._selection_updating || this._initializing || (typeof this.model.select_mode) === 'string' || this.model.select_mode === false)
       return
+    let indices: number[] = []
+    const selected = this.model.source.selected
+    const index: number = row._row.data._index
+    if (e.ctrlKey || e.metaKey) {
+      indices = this.model.source.selected.indices
+    } else if (e.shiftKey && selected.indices.length) {
+      const start = selected.indices[selected.indices.length-1]
+      if (index>start) {
+        for (let i = start; i<index; i++)
+          indices.push(i)
+      } else {
+        for (let i = start; i>index; i--)
+          indices.push(i)
+      }
+    }
+    if (indices.indexOf(index) < 0)
+      indices.push(index)
+    else
+      indices.splice(indices.indexOf(index), 1)
+    // Remove the first selected indices when selectable is an int.
+    if (typeof this.model.select_mode === 'number') {
+      while (indices.length > this.model.select_mode) {
+        indices.shift()
+      }
+    }
+    const filtered = this._filter_selected(indices)
+    this.tabulator.deselectRow()
+    this.tabulator.selectRow(filtered)
     this._selection_updating = true
-    const indices: any = data.map((row: any) => row._index)
-    this.model.source.selected.indices = indices;
+    selected.indices = filtered
+    this._selection_updating = false
+  }
+
+  _filter_selected(indices: number[]): number[] {
+    const filtered = []
+    for (const ind of indices) {
+      if (this.model.selectable_rows == null ||
+          this.model.selectable_rows.indexOf(ind) >= 0)
+        filtered.push(ind)
+    }
+    return filtered
+  }
+
+  rowSelectionChanged(data: any, _: any): void {
+    if (this._selection_updating || this._initializing || (typeof this.model.select_mode) === 'boolean' || (typeof this.model.select_mode) === 'number' || this.model.configuration.dataTree)
+      return
+    const indices: number[] = data.map((row: any) => row._index)
+    const filtered = this._filter_selected(indices)
+    this._selection_updating = indices.length === filtered.length
+    this.model.source.selected.indices = filtered
     this._selection_updating = false
   }
 
@@ -482,6 +666,7 @@ export const TableLayout = Enum("fit_data", "fit_data_fill", "fit_data_stretch",
 export namespace DataTabulator {
   export type Attrs = p.AttrsOf<Props>
   export type Props = HTMLBox.Props & {
+    aggregators: p.Property<any>
     columns: p.Property<TableColumn[]>
     configuration: p.Property<any>
     download: p.Property<boolean>
@@ -491,11 +676,14 @@ export namespace DataTabulator {
     frozen_rows: p.Property<number[]>
     groupby: p.Property<string[]>
     hidden_columns: p.Property<string[]>
+    indexes: p.Property<string[]>
     layout: p.Property<typeof TableLayout["__type__"]>
     max_page: p.Property<number>
     page: p.Property<number>
     page_size: p.Property<number>
     pagination: p.Property<string | null>
+    select_mode: p.Property<any>
+    selectable_rows: p.Property<number[] | null>
     source: p.Property<ColumnDataSource>
     sorters: p.Property<any[]>
     styles: p.Property<any>
@@ -520,6 +708,7 @@ export class DataTabulator extends HTMLBox {
     this.prototype.default_view = DataTabulatorView;
 
     this.define<DataTabulator.Props>(({Any, Array, Boolean, Nullable, Number, Ref, String}) => ({
+      aggregators:    [ Any,                     {} ],
       configuration:  [ Any,                     {} ],
       columns:        [ Array(Ref(TableColumn)), [] ],
       download:       [ Boolean,               true ],
@@ -529,11 +718,14 @@ export class DataTabulator extends HTMLBox {
       frozen_rows:    [ Array(Number),           [] ],
       groupby:        [ Array(String),           [] ],
       hidden_columns: [ Array(String),           [] ],
+      indexes:        [ Array(String),           [] ],
       layout:         [ TableLayout,     "fit_data" ],
       max_page:       [ Number,                   0 ],
       pagination:     [ Nullable(String),      null ],
       page:           [ Number,                   0 ],
       page_size:      [ Number,                   0 ],
+      select_mode:    [ Any,                   true ],
+      selectable_rows: [ Nullable(Array(Number)), null ],
       source:         [ Ref(ColumnDataSource)       ],
       sorters:        [ Array(Any),              [] ],
       styles:         [ Any,                     {} ],
