@@ -2,14 +2,16 @@
 """
 Defines custom VTKPlot bokeh model to render VTK objects.
 """
-from bokeh.core.properties import (String, Bool, Dict, Any, Override,
-                                   Instance, Int, Float, PositiveInt,
-                                   Enum, List)
+from bokeh.core.properties import (
+    String, Bool, Dict, Any, Override, Instance, Int, Float, PositiveInt,
+    Enum, List, Nullable
+)
 from bokeh.core.has_props import abstract
 from bokeh.core.enums import enumeration
 from bokeh.models import HTMLBox, Model, ColorMapper
 
-from ..util import classproperty, bundled_files
+from ..io.resources import bundled_files
+from ..util import classproperty
 
 vtk_cdn = "https://unpkg.com/vtk.js@14.16.4/dist/vtk.js"
 
@@ -56,7 +58,6 @@ class AbstractVTKPlot(HTMLBox):
 
     __js_require__ = {
         "paths": {"vtk": vtk_cdn[:-3]},
-        "exports": {"vtk": None},
         "shim": {
             "vtk": {"exports": "vtk"},
         }
@@ -76,6 +77,9 @@ class AbstractVTKPlot(HTMLBox):
 
     width = Override(default=300)
 
+    annotations = List(Dict(String, Any))
+
+
 
 class VTKSynchronizedPlot(AbstractVTKPlot):
     """
@@ -100,7 +104,7 @@ class VTKJSPlot(AbstractVTKPlot):
     Bokeh model for plotting a 3D scene saved in the `.vtk-js` format
     """
 
-    data = String(help="""The serialized vtk.js data""")
+    data = Nullable(String, help="""The serialized vtk.js data""")
 
     enable_keybindings = Bool(default=False)
 
@@ -117,7 +121,7 @@ class VTKVolumePlot(AbstractVTKPlot):
     controller_expanded = Bool(default=True, help="""
         If True the volume controller panel options is expanded in the view""")
 
-    data = Dict(String, Any)
+    data = Nullable(Dict(String, Any))
 
     diffuse = Float(default=0.7)
 
