@@ -2,8 +2,6 @@
 HoloViews integration for Panel including a Pane to render HoloViews
 objects and their widgets and support for Links
 """
-from __future__ import absolute_import, division, unicode_literals
-
 import sys
 
 from collections import OrderedDict, defaultdict
@@ -84,7 +82,7 @@ class HoloViews(PaneBase):
     _rerender_params = ['object', 'backend']
 
     def __init__(self, object=None, **params):
-        super(HoloViews, self).__init__(object, **params)
+        super().__init__(object, **params)
         self._initialized = False
         self._responsive_content = False
         self._restore_plot = None
@@ -261,7 +259,7 @@ class HoloViews(PaneBase):
 
         kwargs = {p: v for p, v in self.param.get_param_values()
                   if p in Layoutable.param and p != 'name'}
-        child_pane = self._panes.get(backend, Pane)(state, **kwargs)
+        child_pane = self._get_pane(backend, state, **kwargs)
         self._update_plot(plot, child_pane)
         model = child_pane._get_model(doc, root, parent, comm)
         if ref in self._plots:
@@ -272,6 +270,12 @@ class HoloViews(PaneBase):
         self._models[ref] = (model, parent)
         return model
 
+    def _get_pane(self, backend, state, **kwargs):
+        pane_type = self._panes.get(backend, Pane)
+        if isinstance(pane_type, type) and issubclass(pane_type, Matplotlib):
+            kwargs['tight'] = True
+        return pane_type(state, **kwargs)
+
     def _render(self, doc, comm, root):
         import holoviews as hv
         from holoviews import Store, renderer as load_renderer
@@ -291,6 +295,8 @@ class HoloViews(PaneBase):
             params = {}
             if self.theme is not None:
                 params['theme'] = self.theme
+            elif doc.theme and getattr(doc.theme, '_json') != {'attrs': {}}:
+                params['theme'] = doc.theme
             if mode != renderer.mode:
                 params['mode'] = mode
             if params:
@@ -315,7 +321,7 @@ class HoloViews(PaneBase):
             old_plot.cleanup()
         if old_pane:
             old_pane._cleanup(root)
-        super(HoloViews, self)._cleanup(root)
+        super()._cleanup(root)
 
     #----------------------------------------------------------------
     # Public API
@@ -611,9 +617,11 @@ def link_axes(root_view, root_model):
 
             fig = p.state
             if fig.x_range.tags:
-                range_map[fig.x_range.tags[0]].append((fig, p, fig.xaxis[0], fig.x_range))
+                tag = tuple(tuple(t) if isinstance(t, list) else t for t in fig.x_range.tags[0])
+                range_map[tag].append((fig, p, fig.xaxis[0], fig.x_range))
             if fig.y_range.tags:
-                range_map[fig.y_range.tags[0]].append((fig, p, fig.yaxis[0], fig.y_range))
+                tag = tuple(tuple(t) if isinstance(t, list) else t for t in fig.y_range.tags[0])
+                range_map[tag].append((fig, p, fig.yaxis[0], fig.y_range))
 
     for (tag), axes in range_map.items():
         fig, p, ax, axis = axes[0]
@@ -652,7 +660,11 @@ def link_axes(root_view, root_model):
 
             for sp in plots:
                 for callback in sp.callbacks:
-                    if not any(c in callback.models or c in callback.extra_models for c in changed):
+                    models = callback.models
+                    if hasattr(callback, 'extra_models'):
+                        # No more extra_models in HoloViews 2.0
+                        models += callback.extra_models
+                    if not any(c in models for c in changed):
                         continue
                     if 'x_range' in changed:
                         sp.handles['x_range'] = p.handles['x_range']
