@@ -402,6 +402,11 @@ class Param(PaneBase):
             options = p_obj.get_range()
             if not options and value is not None:
                 options = [value]
+            # This applies to widgets whose `options` Parameter is a List type,
+            # such as AutoCompleteInput.
+            if ('options' in widget_class.param
+                and isinstance(widget_class.param['options'], param.List)):
+                options = list(options.values())
             kw['options'] = options
         if hasattr(p_obj, 'get_soft_bounds'):
             bounds = p_obj.get_soft_bounds()
@@ -480,7 +485,11 @@ class Param(PaneBase):
                     self._rerender()
                 return
             elif change.what == 'objects':
-                updates['options'] = p_obj.get_range()
+                options = p_obj.get_range()
+                if ('options' in widget.param and
+                    isinstance(widget.param['options'], param.List)):
+                    options = list(options)
+                updates['options'] = options
             elif change.what == 'bounds':
                 start, end = p_obj.get_soft_bounds()
                 supports_bounds = hasattr(widget, 'start')
