@@ -5,6 +5,7 @@ models rendered on the frontend.
 """
 
 import difflib
+import datetime as dt
 import re
 import sys
 import textwrap
@@ -19,7 +20,7 @@ import param
 
 from bokeh.models import LayoutDOM
 from bokeh.model import DataModel
-from param.parameterized import ParameterizedMetaclass
+from param.parameterized import ParameterizedMetaclass, Watcher
 from tornado import gen
 
 from .config import config
@@ -33,7 +34,7 @@ from .models.reactive_html import (
 from .util import edit_readonly, escape, updating
 from .viewable import Layoutable, Renderable, Viewable
 
-LinkWatcher = namedtuple("Watcher","inst cls fn mode onlychanged parameter_names what queued target links transformed bidirectional_watcher")
+LinkWatcher = namedtuple("Watcher", Watcher._fields+('target', 'links', 'transformed', 'bidirectional_watcher'))
 
 
 class Syncable(Renderable):
@@ -396,7 +397,12 @@ class Reactive(Syncable, Viewable):
                         _reverse_updating.remove(event.name)
             bidirectional_watcher = target.param.watch(reverse_link, list(reverse_links))
 
-        link = LinkWatcher(*tuple(cb)+(target, links, callbacks is not None, bidirectional_watcher))
+        link_args = tuple(cb)
+        # Compatibility with Param versions where precedence is dropped
+        # from iterator for backward compatibility with older Panel versions
+        if 'precedence' in Watcher._fields and len(link_args) < len(Watcher._fields):
+            link_args += (cb.precedence,)
+        link = LinkWatcher(*(link_args+(target, links, callbacks is not None, bidirectional_watcher)))
         self._links.append(link)
         return cb
 
@@ -661,6 +667,13 @@ class SyncableData(Reactive):
         for ref, (m, _) in self._models.items():
             self._apply_update(events, msg, m.source.selected, ref)
 
+    def _apply_stream(self, ref, model, stream, rollover):
+        self._changing[ref] = ['data']
+        try:
+            model.source.stream(stream, rollover)
+        finally:
+            del self._changing[ref]
+
     @updating
     def _stream(self, stream, rollover=None):
         self._processed, _ = self._get_data()
@@ -674,9 +687,16 @@ class SyncableData(Reactive):
                 if comm and 'embedded' not in root.tags:
                     push(doc, comm)
             else:
-                cb = partial(m.source.stream, stream, rollover)
+                cb = partial(self._apply_stream, ref, m, stream, rollover)
                 doc.add_next_tick_callback(cb)
 
+    def _apply_patch(self, ref, model, patch):
+        self._changing[ref] = ['data']
+        try:
+            model.source.patch(patch)
+        finally:
+            del self._changing[ref]
+
     @updating
     def _patch(self, patch):
         for ref, (m, _) in self._models.items():
@@ -689,9 +709,20 @@ class SyncableData(Reactive):
                 if comm and 'embedded' not in root.tags:
                     push(doc, comm)
             else:
-                cb = partial(m.source.patch, patch)
+                cb = partial(self._apply_patch, ref, m, patch)
                 doc.add_next_tick_callback(cb)
 
+    def _update_manual(self, *events):
+        """
+        Skip events triggered internally
+        """
+        processed_events = []
+        for e in events:
+            if e.name == self._data_params[0] and e.type == 'triggered' and self._updating:
+                continue
+            processed_events.append(e)
+        super()._update_manual(*processed_events)
+
     def stream(self, stream_value, rollover=None, reset_index=True):
         """
         Streams (appends) the `stream_value` provided to the existing
@@ -919,8 +950,17 @@ class ReactiveData(SyncableData):
                 k = self._renamed_cols.get(k, k)
                 if isinstance(v, dict):
                     v = [v for _, v in sorted(v.items(), key=lambda it: int(it[0]))]
+                v = np.asarray(v)
+                old_dtype = old_raw[k].dtype
+                if old_dtype.kind == 'M':
+                    v = (v * 10e5).astype(old_raw[k].dtype)
+                elif old_dtype.kind == 'O':
+                    if all(isinstance(ov, dt.date) for ov in old_raw[k]):
+                        v = np.array([dt.date.fromtimestamp(iv/1000) for iv in v])
+                else:
+                    v = v.astype(old_raw[k].dtype)
                 try:
-                    isequal = (old_data[k] == np.asarray(v)).all()
+                    isequal = (old_data[k] == v).all()
                 except Exception:
                     isequal = False
                 if not isequal:
@@ -1194,7 +1234,7 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
 
     _scripts = {}
 
-    _script_assignment = r'data.([^[^\d\W]\w*)[ ]*[\+,\-,\*,\\,%,\*\*,<<,>>,>>>,&,^,|,&&,||,??]*='
+    _script_assignment = r'data\.([^[^\d\W]\w*)[ ]*[\+,\-,\*,\\,%,\*\*,<<,>>,>>>,&,\^,|,\&\&,\|\|,\?\?]*='
 
     __abstract = True
 
@@ -1222,20 +1262,13 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
             else:
                 params[children_param] = panel(child_value)
         super().__init__(**params)
+        self._panes = {}
         self._event_callbacks = defaultdict(lambda: defaultdict(list))
 
     def _cleanup(self, root):
-        for children_param in self._parser.children.values():
-            children = getattr(self, children_param)
-            mode = self._child_config.get(children_param)
-            if mode != 'model':
-                continue
-            if isinstance(children, dict):
-                children = children.values()
-            elif not isinstance(children, list):
-                children = [children]
-            for child in children:
-                child._cleanup(root)
+        for child, panes in self._panes.items():
+            for pane in panes:
+                pane._cleanup(root)
         super()._cleanup(root)
 
     @property
@@ -1260,7 +1293,11 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
         }
         data_params = {}
         for k, v in self.param.get_param_values():
-            if (k in ignored and k != 'name') or ((self.param[k].precedence or 0) < 0):
+            if (
+                (k in ignored and k != 'name') or
+                ((self.param[k].precedence or 0) < 0) or
+                (isinstance(v, Viewable) and not isinstance(self.param[k], param.ClassSelector))
+            ):
                 continue
             if isinstance(v, str):
                 v = bleach.clean(v)
@@ -1297,12 +1334,11 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
                     node_events[e] = False
         return events
 
-    def _get_children(self, doc, root, model, comm, old_children=None):
+    def _get_children(self, doc, root, model, comm):
         from .pane import panel
-        old_children = old_children or {}
         old_models = model.children
         new_models = {parent: [] for parent in self._parser.children}
-        new_panes = {}
+        new_panes, internal_panes = {}, {}
 
         for parent, children_param in self._parser.children.items():
             mode = self._child_config.get(children_param, 'model')
@@ -1319,20 +1355,13 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
             else:
                 panes = [panel(panes)]
             new_panes[parent] = panes
+            if isinstance(panes, dict):
+                panes = list(panes.values())
+            internal_panes[children_param] = panes
 
-        for children_param, old_panes in old_children.items():
-            mode = self._child_config.get(children_param, 'model')
-            if mode == 'literal':
-                continue
-            panes = getattr(self, children_param)
-            if not isinstance(panes, (list, dict)):
-                panes = [panes]
-                old_panes = [old_panes]
-            elif isinstance(panes, dict):
-                panes = panes.values()
-                old_panes = old_panes.values()
+        for children_param, old_panes in self._panes.items():
             for old_pane in old_panes:
-                if old_pane not in panes and hasattr(old_pane, '_cleanup'):
+                if old_pane not in internal_panes.get(children_param, []):
                     old_pane._cleanup(root)
 
         for parent, child_panes in new_panes.items():
@@ -1342,11 +1371,9 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
             mode = self._child_config.get(children_param, 'model')
             if mode == 'literal':
                 new_models[parent] = children_param
-            elif children_param in old_children:
+            elif children_param in self._panes:
                 # Find existing models
-                old_panes = old_children[children_param]
-                if not isinstance(old_panes, (list, dict)):
-                    old_panes = [old_panes]
+                old_panes = self._panes[children_param]
                 for i, pane in enumerate(child_panes):
                     if pane in old_panes and root.ref['id'] in pane._models:
                         child, _ = pane._models[root.ref['id']]
@@ -1361,6 +1388,7 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
                     pane._get_model(doc, root, model, comm)
                     for pane in child_panes
                 ]
+        self._panes = internal_panes
         return self._process_children(doc, root, model, comm, new_models)
 
     def _get_template(self):
@@ -1500,17 +1528,21 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
                     data_msg[prop] = v
             elif prop in list(Reactive.param)+['events']:
                 model_msg[prop] = v
-            elif prop in self.param and (self.param[prop].precedence or 0) < 0:
+            elif (
+                (prop in self.param) and (
+                    ((self.param[prop].precedence or 0) < 0) or
+                    (isinstance(v, Viewable) and not isinstance(self.param[prop], param.ClassSelector))
+                )
+            ):
                 continue
             elif isinstance(v, str):
                 data_msg[prop] = bleach.clean(v)
             else:
                 data_msg[prop] = v
         if new_children:
-            old_children = {key: events[key].old for key in new_children}
             if self._parser.looped:
                 model_msg['html'] = escape(self._get_template())
-            children = self._get_children(doc, root, model, comm, old_children)
+            children = self._get_children(doc, root, model, comm)
         else:
             children = None
         if children is not None:
