@@ -1,12 +1,11 @@
-from __future__ import absolute_import, division, unicode_literals
-
 import os
 import sys
 import pytest
 
 from base64 import b64decode, b64encode
+from pathlib import Path
 
-from panel.pane import GIF, JPG, PNG, SVG
+from panel.pane import GIF, ICO, JPG, PDF, PNG, SVG
 from panel.pane.markup import escape
 from io import BytesIO, StringIO
 
@@ -55,10 +54,13 @@ twopixel = dict(\
           b'QEBAQEBAQEBAQEBAQEBAQH/wAARCAABAAIDAREAAhEBAxEB/8QAFAABAAAAAAAA' + \
           b'AAAAAAAAAAAACf/EABoQAAEFAQAAAAAAAAAAAAAAAAYABAU2dbX/xAAVAQEBAAA' + \
           b'AAAAAAAAAAAAAAAAFBv/EABkRAAEFAAAAAAAAAAAAAAAAAAEAAjFxsf/aAAwDAQ' + \
-          b'ACEQMRAD8AA0qs5HvTHQcJdsChioXSbOr/2Q==')
+          b'ACEQMRAD8AA0qs5HvTHQcJdsChioXSbOr/2Q==',
+    ico = b'AAABAAEAAgEAAAEAIAA0AAAAFgAAACgAAAACAAAAAgAAAAEAIAAAAAAACAAAAHQ' + \
+          b'SAAB0EgAAAAAAAAAAAAD//////////wAAAAA=')
+
 
 def test_imgshape():
-    for t in [PNG, JPG, GIF]:
+    for t in [PNG, JPG, GIF, ICO]:
         w,h = t._imgshape(b64decode(twopixel[t.name.lower()]))
         assert w == 2
         assert h == 1
@@ -72,7 +74,7 @@ def test_load_from_byteio():
         memory.write(image_file.read())
 
     image_pane = PNG(memory)
-    image_data = image_pane._img()
+    image_data = image_pane._data()
     assert b'PNG' in image_data
 
 @pytest.mark.skipif(sys.version_info.major <= 2, reason="Doesn't work with python 2")
@@ -85,23 +87,30 @@ def test_load_from_stringio():
         memory.write(str(image_file.read()))
 
     image_pane = PNG(memory)
-    image_data = image_pane._img()
+    image_data = image_pane._data()
     assert 'PNG' in image_data
 
 def test_loading_a_image_from_url():
     """Tests the loading of a image from a url"""
-    url = 'https://upload.wikimedia.org/wikipedia/commons/7/71/' \
-          '1700_CE_world_map.PNG'
+    url = 'https://file-examples-com.github.io/uploads/2017/10/file_example_PNG_500kB.png'
 
     image_pane = PNG(url)
-    image_data = image_pane._img()
+    image_data = image_pane._data()
+    assert b'PNG' in image_data
+
+
+def test_loading_a_image_from_pathlib():
+    """Tests the loading of a image from a pathlib"""
+    filepath = Path(__file__).parent.parent / "test_data" / "logo.png"
+
+    image_pane = PNG(filepath)
+    image_data = image_pane._data()
     assert b'PNG' in image_data
 
 
 def test_image_alt_text(document, comm):
     """Tests the loading of a image from a url"""
-    url = 'https://upload.wikimedia.org/wikipedia/commons/7/71/' \
-          '1700_CE_world_map.PNG'
+    url = 'https://file-examples-com.github.io/uploads/2017/10/file_example_PNG_500kB.png'
 
     image_pane = PNG(url, embed=False, alt_text="Some alt text")
     model = image_pane.get_root(document, comm)
@@ -111,10 +120,24 @@ def test_image_alt_text(document, comm):
 
 def test_image_link_url(document, comm):
     """Tests the loading of a image from a url"""
-    url = 'https://upload.wikimedia.org/wikipedia/commons/7/71/' \
-          '1700_CE_world_map.PNG'
+    url = 'https://file-examples-com.github.io/uploads/2017/10/file_example_PNG_500kB.png'
 
     image_pane = PNG(url, embed=False, link_url="http://anaconda.org")
     model = image_pane.get_root(document, comm)
 
     assert model.text.startswith('&lt;a href=&quot;http://anaconda.org&quot;')
+
+
+def test_pdf_embed(document, comm):
+    pdf_pane = PDF('https://www.w3.org/WAI/ER/tests/xhtml/testfiles/resources/pdf/dummy.pdf')
+    model = pdf_pane.get_root(document, comm)
+
+    assert model.text.startswith("&lt;embed src=&quot;data:application/pdf;base64,")
+    
+
+def test_pdf_no_embed(document, comm):
+    url = 'https://www.w3.org/WAI/ER/tests/xhtml/testfiles/resources/pdf/dummy.pdf'
+    pdf_pane = PDF(url, embed=False)
+    model = pdf_pane.get_root(document, comm)
+
+    assert model.text.startswith(f"&lt;embed src=&quot;{url}")
