@@ -1,17 +1,17 @@
-from __future__ import print_function
-
 import os
 
 import param
 
 from bokeh.models import (
-    Div, Slider, Select, RangeSlider, MultiSelect, Row as BkRow,
-    CheckboxGroup, Toggle, Button, TextInput as BkTextInput,
-    Tabs as BkTabs, Column as BkColumn, TextInput)
+    Div, Slider, Select, RangeSlider as BkRangeSlider, MultiSelect,
+    Row as BkRow, CheckboxGroup, Toggle, Button, TextInput as
+    BkTextInput,Tabs as BkTabs, Column as BkColumn, TextInput,
+    AutocompleteInput as BkAutocompleteInput,
+)
 from panel.pane import Pane, PaneBase, Matplotlib, Bokeh, HTML
 from panel.layout import Tabs, Row
 from panel.param import Param, ParamMethod, ParamFunction, JSONInit
-from panel.widgets import LiteralInput
+from panel.widgets import AutocompleteInput, LiteralInput, NumberInput, RangeSlider
 from panel.tests.util import mpl_available, mpl_figure
 
 
@@ -221,7 +221,7 @@ def test_range_param(document, comm):
     model = test_pane.get_root(document, comm=comm)
 
     widget = model.children[1]
-    assert isinstance(widget, RangeSlider)
+    assert isinstance(widget, BkRangeSlider)
     assert widget.start == 0
     assert widget.end == 1.1
     assert widget.value == (0.1, 0.5)
@@ -408,6 +408,25 @@ def test_param_precedence(document, comm):
     a_param.precedence = 1
     assert test_pane._widgets['a'] in test_pane._widget_box.objects
 
+    a_param.precedence = None
+    assert test_pane._widgets['a'] in test_pane._widget_box.objects
+
+
+def test_hide_constant(document, comm):
+    class Test(param.Parameterized):
+        a = param.Number(default=1.2, bounds=(0, 5), constant=True)
+
+    test = Test()
+    test_pane = Pane(test, parameters=['a'], hide_constant=True)
+    model = test_pane.get_root(document, comm=comm)
+
+    slider = model.children[1]
+    assert not slider.visible
+
+    test.param.a.constant = False
+
+    assert slider.visible
+
 
 def test_param_label(document, comm):
     class Test(param.Parameterized):
@@ -626,6 +645,45 @@ def test_set_widgets(document, comm):
     assert number.height == 100
     assert isinstance(text, TextInput)
 
+
+def test_set_widgets_throttled(document, comm):
+    class Test(param.Parameterized):
+        a = param.Number(default=0, bounds=(0, 10), precedence=1)
+
+    test = Test()
+    pane = Param(test)
+    model = pane.get_root(document, comm=comm)
+
+    pane.widgets = {"a": {"throttled": False}}
+    assert len(model.children) == 2
+    _, number = model.children
+
+    number.value = 1
+    assert number.value == 1
+    assert number.value_throttled != 1
+    assert test.a == 1
+
+    test.a = 2
+    assert number.value == 2
+    assert number.value_throttled != 2
+    assert test.a == 2
+
+    # By setting throttled to true,
+    # `test.a` is linked to `number.value_throttled`
+    # instead of `number.value`.
+    pane.widgets = {"a": {"throttled": True}}
+    assert len(model.children) == 2
+    _, number = model.children
+
+    pane._widgets['a']._process_events({'value_throttled': 3})
+    assert number.value != 3
+    assert test.a == 3
+
+    pane._widgets['a']._process_events({'value': 4})
+    assert test.a == 3
+    assert number.value == 4
+
+
 def test_set_show_name(document, comm):
     class Test(param.Parameterized):
         a = param.Number(bounds=(0, 10))
@@ -1143,3 +1201,65 @@ def test_change_object_and_keep_parameters():
     view.text = TextModel(text="New TextModel")
     # Then
     assert view.text_pane.parameters==["text"]
+
+
+def test_rerender_bounded_widget_when_bounds_set_and_unset():
+    class Test(param.Parameterized):
+        num = param.Range()
+
+    test = Test()
+    p = Param(test)
+
+    assert isinstance(p._widgets['num'], LiteralInput)
+    assert p._widgets['num'] in p._widget_box
+
+    test.param.num.bounds = (0, 5)
+
+    assert isinstance(p._widgets['num'], RangeSlider)
+    assert p._widgets['num'] in p._widget_box
+
+    test.param.num.bounds = (None, 5)
+
+    assert isinstance(p._widgets['num'], LiteralInput)
+    assert p._widgets['num'] in p._widget_box
+
+
+def test_numberinput_bounds():
+
+    class Test(param.Parameterized):
+        num = param.Number(default=5, bounds=(0, 5))
+
+    test = Test()
+    p = Param(test, widgets={'num': NumberInput})
+
+    numinput = p.layout[1]
+
+    assert numinput.start == 0
+    assert numinput.end == 5
+
+def test_set_widget_autocompleteinput(document, comm):
+
+    class Test(param.Parameterized):
+        # Testing with default='' and check_on_set=False since this feels
+        # like the most sensible default config for Selector -> AutocompleteInput
+        choice = param.Selector(default='', objects=['a', 'b'], check_on_set=False)
+
+    test = Test()
+    test_pane = Param(test, widgets={'choice': AutocompleteInput})
+
+    model = test_pane.get_root(document, comm=comm)
+
+    autocompleteinput = model.children[1]
+    assert isinstance(autocompleteinput, BkAutocompleteInput)
+    assert autocompleteinput.completions == ['a', 'b']
+    assert autocompleteinput.value == ''
+    assert autocompleteinput.disabled == False
+
+    # Check changing param value updates widget
+    test.choice = 'b'
+    assert autocompleteinput.value == 'b'
+
+    # Check changing param attribute updates widget
+    test.param['choice'].objects = ['c', 'd']
+    assert autocompleteinput.completions == ['c', 'd']
+    assert autocompleteinput.value == ''
