@@ -13,14 +13,17 @@ from .indicators import ( # noqa
     LoadingSpinner,
     Number,
     Progress,
-    Trend
+    Trend,
+    Tqdm,
 )
 from .input import (  # noqa
     ColorPicker,
     Checkbox,
     DatetimeInput,
     DatePicker,
+    DatetimePicker,
     DatetimeRangeInput,
+    DatetimeRangePicker,
     FileInput,
     LiteralInput,
     StaticText,
@@ -32,11 +35,12 @@ from .input import (  # noqa
     PasswordInput,
     TextAreaInput,
 )
-from .misc import Audio, FileDownload, VideoStream # noqa
+from .misc import FileDownload, VideoStream # noqa
 from .player import DiscretePlayer, Player # noqa
 from .slider import ( # noqa
-    DateSlider, DateRangeSlider, DiscreteSlider, FloatSlider,
-    IntSlider, IntRangeSlider, RangeSlider
+    DateSlider, DateRangeSlider, DiscreteSlider, EditableRangeSlider,
+    EditableFloatSlider, EditableIntSlider, FloatSlider, IntSlider,
+    IntRangeSlider, RangeSlider
 )
 from .select import ( # noqa
     AutocompleteInput, CheckBoxGroup, CheckButtonGroup, CrossSelector,
@@ -45,4 +49,5 @@ from .select import ( # noqa
 )
 from .speech_to_text import SpeechToText, Grammar, GrammarList # noqa
 from .tables import DataFrame, Tabulator  # noqa
+from .terminal import Terminal # noqa
 from .text_to_speech import TextToSpeech, Utterance, Voice # noqa
