@@ -3,8 +3,6 @@
 Defines a FileSelector widget which allows selecting files and
 directories on the server.
 """
-from __future__ import absolute_import, division, unicode_literals
-
 import os
 
 from collections import OrderedDict
@@ -12,6 +10,7 @@ from fnmatch import fnmatch
 
 import param
 
+from ..io import PeriodicCallback
 from ..layout import Column, Divider, Row
 from ..viewable import Layoutable
 from .base import CompositeWidget
@@ -78,6 +77,14 @@ class FileSelector(CompositeWidget):
         The number of options shown at once (note this is the only
         way to control the height of this widget)""")
 
+    refresh_period = param.Integer(default=None, doc="""
+        If set to non-None value indicates how frequently to refresh
+        the directory contents in milliseconds.""")
+
+    root_directory = param.String(default=None, doc="""
+        If set, overrides directory parameter as the root directory
+        beyond which users cannot navigate.""")
+
     value = param.List(default=[], doc="""
         List of selected files.""")
 
@@ -87,10 +94,13 @@ class FileSelector(CompositeWidget):
         from ..pane import Markdown
         if directory is not None:
             params['directory'] = os.path.abspath(os.path.expanduser(directory))
+        if 'root_directory' in params:
+            root = params['root_directory']
+            params['root_directory'] = os.path.abspath(os.path.expanduser(root))
         if params.get('width') and params.get('height') and 'sizing_mode' not in params:
             params['sizing_mode'] = None
 
-        super(FileSelector, self).__init__(**params)
+        super().__init__(**params)
 
         # Set up layout
         layout = {p: getattr(self, p) for p in Layoutable.param
@@ -102,9 +112,10 @@ class FileSelector(CompositeWidget):
         self._forward = Button(name='▶', width=25, margin=(5, 10), disabled=True)
         self._up = Button(name='⬆', width=25, margin=(5, 10), disabled=True)
         self._directory = TextInput(value=self.directory, margin=(5, 10), width_policy='max')
-        self._go = Button(name='⬇', disabled=True, width=25, margin=(5, 15, 0, 0))
+        self._go = Button(name='⬇', disabled=True, width=25, margin=(5, 10, 0, 0))
+        self._reload = Button(name='↻', width=25, margin=(5, 15, 0, 10))
         self._nav_bar = Row(
-            self._back, self._forward, self._up, self._directory, self._go,
+            self._back, self._forward, self._up, self._directory, self._go, self._reload,
             **dict(layout, width=None, margin=0, width_policy='max')
         )
         self._composite[:] = [self._nav_bar, Divider(margin=0), self._selector]
@@ -122,12 +133,29 @@ class FileSelector(CompositeWidget):
         self.link(self._directory, directory='value')
         self._selector.param.watch(self._update_value, 'value')
         self._go.on_click(self._update_files)
+        self._reload.on_click(self._update_files)
         self._up.on_click(self._go_up)
         self._back.on_click(self._go_back)
         self._forward.on_click(self._go_forward)
         self._directory.param.watch(self._dir_change, 'value')
         self._selector._lists[False].param.watch(self._select, 'value')
         self._selector._lists[False].param.watch(self._filter_blacklist, 'options')
+        self._periodic = PeriodicCallback(callback=self._refresh, period=self.refresh_period or 0)
+        self.param.watch(self._update_periodic, 'refresh_period')
+        if self.refresh_period:
+            self._periodic.start()
+
+    def _update_periodic(self, event):
+        if event.new:
+            self._periodic.period = event.new
+            if not self._periodic.running:
+                self._periodic.start()
+        elif self._periodic.running:
+            self._periodic.stop()
+
+    @property
+    def _root_directory(self):
+        return self.root_directory or self.directory
 
     def _update_value(self, event):
         value = [v for v in event.new if not self.only_files or os.path.isfile(v)]
@@ -136,16 +164,22 @@ class FileSelector(CompositeWidget):
 
     def _dir_change(self, event):
         path = os.path.abspath(os.path.expanduser(self._directory.value))
-        if not path.startswith(self.directory):
-            self._directory.value = self.directory
+        if not path.startswith(self._root_directory):
+            self._directory.value = self._root_directory
             return
         elif path != self._directory.value:
             self._directory.value = path
         self._go.disabled = path == self._cwd
 
-    def _update_files(self, event=None):
+    def _refresh(self):
+        self._update_files(refresh=True)
+
+    def _update_files(self, event=None, refresh=False):
         path = os.path.abspath(self._directory.value)
-        if not os.path.isdir(path):
+        refresh = refresh or (event and getattr(event, 'obj', None) is self._reload)
+        if refresh:
+            path = self._cwd
+        elif not os.path.isdir(path):
             self._selector.options = ['Entered path is not valid']
             self._selector.disabled = True
             return
@@ -154,8 +188,9 @@ class FileSelector(CompositeWidget):
             self._position += 1
 
         self._cwd = path
-        self._go.disabled = True
-        self._up.disabled = path == self.directory
+        if not refresh:
+            self._go.disabled = True
+        self._up.disabled = path == self._root_directory
         if self._position == len(self._stack)-1:
             self._forward.disabled = True
         if 0 <= self._position and len(self._stack) > 1:
@@ -168,7 +203,7 @@ class FileSelector(CompositeWidget):
             if os.path.isdir(check):
                 dirs.append(s)
             elif os.path.isfile(check):
-                dirs.append(s)
+                files.append(s)
 
         paths = [p for p in sorted(dirs)+sorted(files)
                  if self.show_hidden or not os.path.basename(p).startswith('.')]
@@ -220,5 +255,5 @@ class FileSelector(CompositeWidget):
 
     def _go_up(self, event=None):
         path = self._cwd.split(os.path.sep)
-        self._directory.value = os.path.sep.join(path[:-1])
+        self._directory.value = os.path.sep.join(path[:-1]) or os.path.sep
         self._update_files(True)
