@@ -174,6 +174,8 @@ class BaseTable(ReactiveData, Widget):
             title = self.titles.get(col, str(col))
             if col in indexes and len(indexes) > 1 and self.hierarchical:
                 title = 'Index: %s' % ' | '.join(indexes)
+            elif col in self.indexes and col.startswith('level_'):
+                title = ''
             column = TableColumn(field=str(col), title=title,
                                  editor=editor, formatter=formatter,
                                  **col_kwargs)
@@ -324,7 +326,16 @@ class BaseTable(ReactiveData, Widget):
         df = self._filter_dataframe(self.value)
         if df is None:
             return [], {}
-        elif len(self.indexes) > 1:
+        import pandas as pd
+        if isinstance(self.value.index, pd.MultiIndex):
+            indexes = [
+                f'level_{i}' if n is None else n
+                for i, n in enumerate(df.index.names)
+            ]
+        else:
+            default_index = ('level_0' if 'index' in df.columns else 'index')
+            indexes = [df.index.name or default_index]
+        if len(indexes) > 1:
             df = df.reset_index()
         data = ColumnDataSource.from_df(df).items()
         return df, {k if isinstance(k, str) else str(k): v for k, v in data}
@@ -342,8 +353,12 @@ class BaseTable(ReactiveData, Widget):
         if self.value is None or not self.show_index:
             return []
         elif isinstance(self.value.index, pd.MultiIndex):
-            return list(self.value.index.names)
-        return [self.value.index.name or 'index']
+            return [
+                f'level_{i}' if n is None else n
+                for i, n in enumerate(self.value.index.names)
+            ]
+        default_index = ('level_0' if 'index' in self.value.columns else 'index')
+        return [self.value.index.name or default_index]
 
     def stream(self, stream_value, rollover=None, reset_index=True):
         """
@@ -724,16 +739,16 @@ class Tabulator(BaseTable):
                                       objects=['local', 'remote'])
 
     page = param.Integer(default=1, doc="""
-        Currently selected page (indexed starting at 1).""")
+        Currently selected page (indexed starting at 1), if pagination is enabled.""")
 
     page_size = param.Integer(default=20, bounds=(1, None), doc="""
-        Number of rows to render per page.""")
+        Number of rows to render per page, if pagination is enabled.""")
 
     row_height = param.Integer(default=30, doc="""
         The height of each table row.""")
 
-    selectable = param.ObjectSelector(
-        default=True, objects=[True, False, 'checkbox', 'checkbox-single', 'toggle'], doc="""
+    selectable = param.ClassSelector(
+        default=True, class_=(bool, str, int), doc="""
         Defines the selection mode of the Tabulator.
 
           - True
@@ -747,6 +762,8 @@ class Tabulator(BaseTable):
               Same as 'checkbox' but header does not alllow select/deselect all
           - 'toggle'
               Selection toggles when clicked
+          - int
+              The maximum number of selectable rows.
         """)
 
     selectable_rows = param.Callable(default=None, doc="""
@@ -842,11 +859,22 @@ class Tabulator(BaseTable):
     def _get_data(self):
         if self.pagination != 'remote' or self.value is None:
             return super()._get_data()
+        import pandas as pd
         df = self._filter_dataframe(self.value)
         df = self._sort_df(df)
         nrows = self.page_size
         start = (self.page-1)*nrows
         page_df = df.iloc[start: start+nrows]
+        if isinstance(self.value.index, pd.MultiIndex):
+            indexes = [
+                f'level_{i}' if n is None else n
+                for i, n in enumerate(df.index.names)
+            ]
+        else:
+            default_index = ('level_0' if 'index' in df.columns else 'index')
+            indexes = [df.index.name or default_index]
+        if len(indexes) > 1:
+            page_df = page_df.reset_index()
         data = ColumnDataSource.from_df(page_df).items()
         return df, {k if isinstance(k, str) else str(k): v for k, v in data}
 
@@ -855,7 +883,7 @@ class Tabulator(BaseTable):
         return len(self._processed)
 
     def _get_style_data(self):
-        if self.value is None:
+        if self.value is None or self.style is None:
             return {}
         df = self._processed
         if self.pagination == 'remote':
@@ -866,6 +894,8 @@ class Tabulator(BaseTable):
             styler = df.style
         except Exception:
             return {}
+        if styler is None:
+            return {}
         styler._todo = self.style._todo
         styler._compute()
         offset = len(self.indexes) + int(self.selectable in ('checkbox', 'checkbox-single'))
@@ -982,11 +1012,8 @@ class Tabulator(BaseTable):
         nrows = self.page_size
         start = (self.page-1)*nrows
         end = start+nrows
-        if self.sorters:
-            index = self._processed.iloc[start:end].index.values
-            self.value[column].loc[index] = array
-        else:
-            self.value[column].iloc[start:end] = array
+        index = self._processed.iloc[start:end].index.values
+        self.value[column].loc[index] = array
 
     def _update_selection(self, indices):
         if self.pagination != 'remote':
@@ -1044,7 +1071,7 @@ class Tabulator(BaseTable):
     def _get_model(self, doc, root=None, parent=None, comm=None):
         if self._widget_type is None:
             self._widget_type = lazy_load(
-                'panel.models.tabulator', 'DataTabulator', isinstance(comm, JupyterComm)
+                'panel.models.tabulator', 'DataTabulator', isinstance(comm, JupyterComm), root
             )
         if comm:
             with set_resource_mode('inline'):
@@ -1089,9 +1116,13 @@ class Tabulator(BaseTable):
                     column_objs.remove(cols[0])
         ordered += column_objs
 
+        grouping = {
+            group: [str(gc) for gc in group_cols]
+            for group, group_cols in self.groups.items()
+        }
         for i, column in enumerate(ordered):
             matching_groups = [
-                group for group, group_cols in self.groups.items()
+                group for group, group_cols in grouping.items()
                 if column.field in group_cols
             ]
             col_dict = {'field': column.field}
@@ -1191,7 +1222,7 @@ class Tabulator(BaseTable):
             text_kwargs['name'] = 'Filename'
         if 'value' not in text_kwargs:
             text_kwargs['value'] = 'table.csv'
-        filename = TextInput(name='Filename', value='table.csv')
+        filename = TextInput(**text_kwargs)
         filename.jscallback({'table': self}, value="""
         table.filename = cb_obj.value
         """)
